/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.ImageDiffer;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import org.gfxtest.framework.Log;

/**
 * This class contains useful functions (mainly static methods) for RGBA images manipulation.
 *
 * @author Pavel Tisnovsky
 */
public class ImageUtils
{
    final static int aMask = 0xFF000000;
    final static int rMask = 0x00FF0000;
    final static int gMask = 0x0000FF00;
    final static int bMask = 0x000000FF;
    final static int rShift = 16;
    final static int gShift = 8;
    final static int bShift = 0;

    static Log log = new Log(ImageUtils.class.getName(), true);

    public static int colorComponent(int color, int mask, int shift)
    {
        return (color & mask) >> shift;
    }

    public static int getR(int color)
    {
        return colorComponent(color, rMask, rShift);
    }

    public static int getG(int color)
    {
        return colorComponent(color, gMask, gShift);
    }

    public static int getB(int color)
    {
        return colorComponent(color, bMask, bShift);
    }

    public static int compare(int color1, int color2)
    {
        int d1 = Math.abs(getR(color1) - getR(color2));
        int d2 = Math.abs(getG(color1) - getG(color2));
        int d3 = Math.abs(getB(color1) - getB(color2));
        return Math.max(d1, Math.max(d2, d3));
    }

    public static Color rgb2grayscale(int color)
    {
        int r = getR(color);
        int g = getG(color);
        int b = getB(color);
        float gray = (0.3f * r + 0.59f * g + 0.11f * b) / 255.0f;
        return new Color(gray, gray, gray);
    }

    private static BufferedImage readImage(File directory, String fileName) throws IOException
    {
        File imageFile = new File(directory, fileName);
        return ImageIO.read(imageFile);
    }

    public static void writeImage(BufferedImage img, File directory, String fileName) throws IOException
    {
        ImageIO.write(img, "png", new File(directory, fileName));
    }

    public static BufferedImage[] readSourceImages(File firstDirectory, File secondDirectory, String fileName) throws Exception
    {
        BufferedImage[] images = new BufferedImage[2];
        for (int i = 0; i < 2; i++)
        {
            File directory = i==0 ? firstDirectory : secondDirectory;
            try
            {
                images[i] = ImageUtils.readImage(directory, fileName);
                log.logProcess("loading image %s", new File(directory, fileName).getAbsolutePath());
            }
            catch (IOException e)
            {
                throw new Exception("Cannot load or initialize image " + fileName);
            }
        }
        return images;
    }
    
    /**
     * Return new scaled image. When the image is scaled up, nearest neighbor
     * algorithm is used to not to "hide" one-pixel errors.
     * 
     * @param inputImage source image to be scaled
     * @param scale scale factor
     * @return new scaled image
     */
    public static BufferedImage scaleImage(BufferedImage inputImage, float scale)
    {
        int width = (int) Math.ceil(inputImage.getWidth() * scale);
        int height = (int) Math.ceil(inputImage.getHeight() * scale);
        BufferedImage scaledImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D graphics2D = scaledImage.createGraphics();
        graphics2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
        graphics2D.drawImage(inputImage, 0, 0, width, height, null);
        graphics2D.dispose();
        return scaledImage;
    }

}
