/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012, 2013  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.common;

import java.util.HashMap;
import java.util.Map;

import org.gfxtest.framework.Log;
import org.gfxtest.framework.ParameterNotFoundException;



/**
 * This class is used to store actual configuration of graphics test framework.
 * Configuration can be changed via various command line switches.
 * 
 * @author Pavel Tisnovsky
 */
public abstract class Configuration
{
    /**
     * Instance of logger class.
     */
    protected Log log;

    /**
     * Constructor called with an array containing command line arguments.
     * 
     * @param args
     *            command line arguments
     * @param log
     *            instance of logger class
     * @throws ConfigurationException
     *             in case configuration is wrong
     */
    public Configuration(String[] args, Log log) throws ConfigurationException
    {
        this.log = log;
        log.logBegin("read configuration"); //$NON-NLS-1$
        readCommandLineParameters(args);
        log.logEnd("read configuration"); //$NON-NLS-1$
        printParameters();
    }

    /**
     * Implementation of this abstract method should reads and process all
     * command line parameters.
     * 
     * @param args
     *            command line arguments
     */
    protected abstract void readCommandLineParameters(String[] args) throws ConfigurationException;

    /**
     * Implementation of this abstract method should prints all test suite
     * parameters.
     */
    protected abstract void printParameters();

    /**
     * Method for get value of parameter which has an integer type.
     * 
     * @param options
     *            map containing all configuration parameters
     * @param parameterName
     *            parameter name
     * @return integer value of selected parameter
     * @throws ConfigurationException
     *             thrown if parameter is not found or has wrong value
     */
    protected int getIntegerParameter(Map<String, String> options, String parameterName) throws ConfigurationException
    {
        // check if parameter map contains parameter with given name
        if (!options.containsKey(parameterName))
        {
            throw new ParameterNotFoundException(parameterName);
        }
        // get a value of a parameter
        String parameterValue = options.get(parameterName);
        // and parse it as integer
        try
        {
            return Integer.parseInt(parameterValue);
        }
        catch (NumberFormatException e)
        {
            throw new InvalidParameterValueException(parameterName, parameterValue);
        }
    }

    /**
     * Method for get value of parameter which has a string type.
     * 
     * @param options
     *            map containing all configuration parameters
     * @param parameterName
     *            parameter name
     * @return string value of selected parameter
     * @throws ConfigurationException
     *             thrown if parameter is not found or has wrong value
     */
    protected String getStringParameter(Map<String, String> options, String parameterName)
                    throws ConfigurationException
    {
        // check if parameter map contains parameter with given name
        if (!options.containsKey(parameterName))
        {
            throw new ParameterNotFoundException(parameterName);
        }
        return options.get(parameterName);
    }

    /**
     * Resolve all configuration options, ie. parse it and push it to a map.
     * 
     * @param args
     *            command line arguments
     * @return map containing all parameters
     */
    @SuppressWarnings("nls")
    protected Map<String, String> resolveAllOptions(String[] args)
    {
        Map<String, String> options = new HashMap<String, String>();
        // loop over all command line arguments
        for (String arg : args)
        {
            // quick and dirty parsing of command line parameters
            // param is split at '=' symbol
            String[] splittedArg = arg.split("=");
            if (splittedArg.length >= 1 && splittedArg[0].length() >= 2)
            {
                this.log.log("found option " + arg);
                // get parameter name
                String optionName = splittedArg[0].substring(1);
                // get parameter value
                String optionValue = splittedArg.length == 2 ? splittedArg[1] : "true";
                options.put(optionName, optionValue);
            }
            else
            {
                this.log.logError("unknown option " + arg);
            }
        }
        return options;
    }

}
