/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.gfxtest.testsuites;

import java.awt.Graphics2D;



import org.gfxtest.framework.CommonClippingOperations;
import org.gfxtest.framework.CommonPathsGenerator;
import org.gfxtest.framework.CommonRenderingStyles;
import org.gfxtest.framework.GfxTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



/**
 * This test check if clipping is working correctly for a path which is clipped
 * by a round rectangle shape. Various path and various rendering styles are used for
 * rendering.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.RENDER_TEST)
@Transformation(Transformations.NONE)
@GraphicsPrimitive(GraphicsPrimitives.PATH)
@Zoom(1)
public class ClippingPathByRoundRectangleShape extends GfxTest
{
    private static final float[] DOTTED_PATTERN = new float[] {1,40};

    /**
     * Prepare canvas for the rendering.
     *
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void basicSetupForRendering(TestImage image, Graphics2D graphics2d)
    {
        // create clip area
        CommonClippingOperations.createClipUsingRoundRectangleShape(image, graphics2d);
    }

    /**
     * Draw path consisting of one line only clipped by a round rectangle shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawLinePathClippedByRoundRectangleShape(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the line path
        graphics2d.draw(CommonPathsGenerator.createLinePathFloat(image));
    }

    /**
     * Draw quadratic path clipped by a round rectangle shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawQuadraticPathClippedByRoundRectangleShape(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the quadratic path
        graphics2d.draw(CommonPathsGenerator.createQuadraticPathFloat(image));
    }

    /**
     * Draw cubic path clipped by a round rectangle shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCubicPathClippedByRoundRectangleShape(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the cubic path
        graphics2d.draw(CommonPathsGenerator.createCubicPathFloat(image));
    }

    /**
     * Draw closed path clipped by a round rectangle shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawClosedPathClippedByRoundRectangleShape(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the closed path
        graphics2d.draw(CommonPathsGenerator.createClosedPathFloat(image));
    }

    /**
     * Draw crossed path clipped by a round rectangle shape.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     */
    private static void drawCrossedClosedPathClippedByRoundRectangleShape(TestImage image, Graphics2D graphics2d)
    {
        // prepare canvas for the rendering
        basicSetupForRendering(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // draw the crossed path
        graphics2d.draw(CommonPathsGenerator.createCrossedClosedPathFloat(image));
    }

    /**
     * Check if line path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRoundRectangleShapeStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRoundRectangleShapeStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRoundRectangleShapeStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRoundRectangleShapeStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRoundRectangleShapeStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRoundRectangleShapeZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRoundRectangleShapeZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRoundRectangleShapeZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRoundRectangleShapeZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRoundRectangleShapeZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a round rectangle shape. Path is rendered
     * using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRoundRectangleShapeThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render rectangle which is used as a clip shape
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic Bezier path could be clipped by a round rectangle shape. Path
     * is rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRoundRectangleShapeThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render rectangle which is used as a clip shape
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic Bezier path could be clipped by a round rectangle shape. Path
     * is rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRoundRectangleShapeThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render rectangle which is used as a clip shape
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a round rectangle shape. Path
     * is rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRoundRectangleShapeThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render rectangle which is used as a clip shape
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a round rectangle shape. Path
     * is rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRoundRectangleShapeThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render rectangle which is used as a clip shape
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a round rectangle shape. Path is rendered
     * using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRoundRectangleShapeExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render rectangle which is used as a clip shape
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic Bezier path could be clipped by a round rectangle shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRoundRectangleShapeExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render rectangle which is used as a clip shape
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic Bezier path could be clipped by a round rectangle shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRoundRectangleShapeExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render rectangle which is used as a clip shape
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a round rectangle shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRoundRectangleShapeExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render rectangle which is used as a clip shape
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a round rectangle shape. Path
     * is rendered using stroke paint with extra thick width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRoundRectangleShapeExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render rectangle which is used as a clip shape
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRoundRectangleShapeDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRoundRectangleShapeDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRoundRectangleShapeDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRoundRectangleShapeDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed crossed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with default stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRoundRectangleShapeDashedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRoundRectangleShapeDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRoundRectangleShapeDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRoundRectangleShapeDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRoundRectangleShapeDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRoundRectangleShapeDashedThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRoundRectangleShapeDashedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRoundRectangleShapeDashedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRoundRectangleShapeDashedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRoundRectangleShapeDashedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with thick stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRoundRectangleShapeDashedExtraThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRoundRectangleShapeDashedZeroThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawLinePathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRoundRectangleShapeDashedZeroThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawQuadraticPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRoundRectangleShapeDashedZeroThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCubicPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRoundRectangleShapeDashedZeroThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a round rectangle shape. Path is
     * rendered using stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRoundRectangleShapeDashedZeroThickStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a round rectangle shape. Path is rendered
     * using dotted stroke paint with normal stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRoundRectangleShapeDottedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        //CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawLinePathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a round rectangle shape. Path is rendered
     * using dotted stroke paint with normal stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRoundRectangleShapeDottedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        //CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawQuadraticPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a round rectangle shape. Path is rendered
     * using dotted stroke paint with normal stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRoundRectangleShapeDottedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        //CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCubicPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a round rectangle shape. Path is rendered
     * using dotted stroke paint with normal stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRoundRectangleShapeDottedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        //CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a round rectangle shape. Path is rendered
     * using dotted stroke paint with normal stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRoundRectangleShapeDottedStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        //CommonRenderingStyles.setStrokeThickWidth(graphics2d, BasicStroke.CAP_ROUND);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if line path could be clipped by a round rectangle shape. Path is rendered
     * using dotted stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipLinePathByRoundRectangleShapeDottedZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawLinePathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if quadratic path could be clipped by a round rectangle shape. Path is rendered
     * using dotted stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipQuadraticPathByRoundRectangleShapeDottedZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawQuadraticPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if cubic path could be clipped by a round rectangle shape. Path is rendered
     * using dotted stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCubicPathByRoundRectangleShapeDottedZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCubicPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if closed path could be clipped by a round rectangle shape. Path is rendered
     * using dotted stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipClosedPathByRoundRectangleShapeDottedZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Check if crossed closed path could be clipped by a round rectangle shape. Path is rendered
     * using dotted stroke paint with zero stroke width.
     * 
     * @param image
     *            work image
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testClipCrossedClosedPathByRoundRectangleShapeDottedZeroStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // render clip round rectangle
        CommonClippingOperations.renderClipRoundRectangle(image, graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeZeroThick(graphics2d);
        // set dashed pattern
        CommonRenderingStyles.setDashedStrokePattern(graphics2d, DOTTED_PATTERN);
        // set clip region and draw the path
        drawCrossedClosedPathClippedByRoundRectangleShape(image, graphics2d);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     * 
     * @param args
     *            graphics test configuration
     */
    public static void main(String[] args)
    {
        new ClippingPathByRoundRectangleShape().runTestSuite(args);
    }
}
